@@ -2,10 +2,10 @@
 
 import re, abc
 
-from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.db import connection
 from django.db.models import Q
+from django.utils.encoding import force_unicode
 
 from watson.models import SearchEntry, has_int_pk
 
@@ -15,6 +15,14 @@ def regex_from_word(word):
     return u"(\s{word})|(^{word})".format(
         word = re.escape(word),
     )
+    
+    
+def make_escaper(badchars):
+    """Creates an efficient escape function that strips the given characters from the string."""
+    translation_table = dict((ord(c), None) for c in badchars)
+    def escaper(text):
+        return force_unicode(text, errors="ignore").translate(translation_table)
+    return escaper
 
 
 class SearchBackend(object):
@@ -66,10 +74,6 @@ class SearchBackend(object):
     def do_filter(self, engine_slug, queryset, search_text):
         """Filters the given queryset according the the search logic for this backend."""
         raise NotImplementedError
-    
-    def save_search_entry(self, search_entry, obj, adapter):
-        """Saves the given search entry in the database."""
-        search_entry.save()
 
 
 class RegexSearchMixin(object):
@@ -150,6 +154,9 @@ class RegexSearchBackend(RegexSearchMixin, SearchBackend):
     """A search backend that works with SQLite3."""
 
 
+escape_postgres_query_chars = make_escaper(u"():|!&*")
+
+
 class PostgresSearchBackend(SearchBackend):
 
     """A search backend that uses native PostgreSQL full text indices."""
@@ -157,9 +164,9 @@ class PostgresSearchBackend(SearchBackend):
     def escape_postgres_query(self, text):
         """Escapes the given text to become a valid ts_query."""
         return u" & ".join(
-            u"{}:*".format(word)
+            u"{0}:*".format(word)
             for word
-            in text.replace(u"(", u"").replace(u")", u"").replace(u":", u"").replace(u"|", u"").replace("!", "").split()
+            in escape_postgres_query_chars(text).split()
         )
     
     def is_installed(self):
@@ -288,7 +295,7 @@ class PostgresLegacySearchBackend(PostgresSearchBackend):
     
     def escape_postgres_query(self, text):
         """Escapes the given text to become a valid ts_query."""
-        return u" & ".join(text.replace(u"(", u"").replace(u")", u"").replace(u":", u"").replace(u"|", u"").replace("!", "").split())
+        return u" & ".join(escape_postgres_query_chars(text).split())
 
 
 class PostgresPrefixLegacySearchBackend(RegexSearchMixin, PostgresLegacySearchBackend):
@@ -302,16 +309,17 @@ class PostgresPrefixLegacySearchBackend(RegexSearchMixin, PostgresLegacySearchBa
     """
         
 
+escape_mysql_boolean_query_chars = make_escaper(u"+-<>()*\"")
+
 def escape_mysql_boolean_query(search_text):
     return u" ".join(
         u'+{word}*'.format(
             word = word,
         )
-        for word in search_text.replace(u"+", u"").replace(u"-", u"").replace(u"<", u"").replace(u">", u"").replace(u"(", u"").replace(u")", u"").replace(u"*", u"").replace(u'"', u"").split()
+        for word in escape_mysql_boolean_query_chars(search_text).split()
     )
     
 
-        
 class MySQLSearchBackend(SearchBackend):
 
     def is_installed(self):
@@ -414,6 +422,21 @@ class MySQLSearchBackend(SearchBackend):
             select_params = (search_text, search_text, search_text,),
             order_by = ("-watson_rank",),
         )
+
+
+def get_postgresql_version(connection):
+    """Returns the version number of the PostgreSQL connection."""
+    try:
+        from django.db.backends.postgresql.version import get_version  # Django 1.3
+    except ImportError:
+        # Use the Django 1.4 method.
+        from django.db.backends.postgresql_psycopg2.version import get_version
+        return get_version(connection)
+    else:
+        # Use the Django 1.3 method. 
+        cursor = connection.cursor()
+        major, major2, minor = get_version(cursor)
+        return major * 10000 + major2 * 100 + minor
         
         
 class AdaptiveSearchBackend(SearchBackend):
@@ -425,15 +448,12 @@ class AdaptiveSearchBackend(SearchBackend):
     
     def __new__(cls):
         """Guess the correct search backend and initialize it."""
-        database_engine = settings.DATABASES["default"]["ENGINE"]
-        if database_engine.endswith("postgresql_psycopg2") or database_engine.endswith("postgresql"):
-            from django.db.backends.postgresql.version import get_version
-            cursor = connection.cursor()
-            version = get_version(cursor)
-            if version > (8, 4, 0):
+        if connection.vendor == "postgresql":
+            version = get_postgresql_version(connection)
+            if version > 80400:
                 return PostgresSearchBackend()
-            if version > (8, 3, 0):
+            if version > 80300:
                 return PostgresLegacySearchBackend()
-        if database_engine.endswith("mysql"):
+        if connection.vendor == "mysql":
             return MySQLSearchBackend()
         return RegexSearchBackend()
\ No newline at end of file
