@@ -1,12 +1,13 @@
 """Adapters for registering models with django-watson."""
 
 import sys
+from itertools import chain
 from threading import local
 from functools import wraps
 from weakref import WeakValueDictionary
 
 from django.conf import settings
-from django.core.signals import request_started, request_finished
+from django.core.signals import request_finished
 from django.core.exceptions import ImproperlyConfigured
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
@@ -15,6 +16,7 @@ from django.db.models.query import QuerySet
 from django.db.models.signals import post_save, pre_delete
 from django.utils.html import strip_tags
 from django.utils.importlib import import_module
+from django.utils import simplejson as json
 
 from watson.models import SearchEntry, has_int_pk
 
@@ -155,16 +157,25 @@ class SearchContextError(Exception):
     """Something went wrong with the search context management."""
 
 
+def _bulk_save_search_entries(search_entries):
+    """Creates the given search entry data in the most efficient way possible."""
+    if search_entries:
+        if hasattr(SearchEntry.objects, "bulk_create"):
+            search_entries = list(search_entries)
+            SearchEntry.objects.bulk_create(search_entries)
+        else:
+            for search_entry in search_entries:
+                search_entry.save()
+
+
 class SearchContextManager(local):
 
     """A thread-local context manager used to manage saving search data."""
     
     def __init__(self):
         """Initializes the search context."""
-        self._request_active = False
         self._stack = []
         # Connect to the signalling framework.
-        request_started.connect(self._request_started_receiver)
         request_finished.connect(self._request_finished_receiver)
     
     def is_active(self):
@@ -204,8 +215,7 @@ class SearchContextManager(local):
         # Save all the models.
         tasks, is_invalid = self._stack.pop()
         if not is_invalid:
-            for engine, obj in tasks:
-                engine.update_obj_index(obj)
+            _bulk_save_search_entries(list(chain.from_iterable(engine._update_obj_index_iter(obj) for engine, obj in tasks)))
     
     # Context management.
             
@@ -219,22 +229,14 @@ class SearchContextManager(local):
     
     # Signalling hooks.
         
-    def _request_started_receiver(self, **kwargs):
-        """Signal handler for when the request starts."""
-        self._request_active = True
-        self.start()
-        
     def _request_finished_receiver(self, **kwargs):
-        """Signal handler for when the request ends."""
-        if self._request_active:
-            self._request_active = False
+        """
+        Called at the end of a request, ensuring that any open contexts
+        are closed. Not closing all active contexts can cause memory leaks
+        and weird behaviour.
+        """
+        while self.is_active():
             self.end()
-        # Check for any hanging search contexts.
-        if self.is_active():
-            raise SearchContextError(
-                "Request finished with an open search context. All calls to search_context_manager.begin() "
-                "should be balanced by a call to search_context_manager.end()."
-            )
             
             
 class SearchContext(object):
@@ -355,7 +357,7 @@ class SearchEngine(object):
         del self._registered_models[model]
         # Disconnect from the signalling framework.
         post_save.disconnect(self._post_save_receiver, model)
-        pre_delete.connect(self._pre_delete_receiver, model)
+        pre_delete.disconnect(self._pre_delete_receiver, model)
         
     def get_registered_models(self):
         """Returns a sequence of models that have been registered with this search engine."""
@@ -393,35 +395,40 @@ class SearchEngine(object):
             )
         return object_id_int, search_entries
     
-    def update_obj_index(self, obj):
-        """Updates the search index for the given obj."""
+    def _update_obj_index_iter(self, obj):
+        """Either updates the given object index, or yields an unsaved search entry."""
         model = obj.__class__
         adapter = self.get_adapter(model)
         content_type = ContentType.objects.get_for_model(model)
         object_id = unicode(obj.pk)
+        # Create the search entry data.
+        search_entry_data = {
+            "engine_slug": self._engine_slug,
+            "title": adapter.get_title(obj),
+            "description": adapter.get_description(obj),
+            "content": adapter.get_content(obj),
+            "url": adapter.get_url(obj),
+            "meta_encoded": json.dumps(adapter.get_meta(obj)),
+        }
         # Try to get the existing search entry.
         object_id_int, search_entries = self._get_entries_for_obj(obj)
-        try:
-            search_entry = search_entries.get()
-        except SearchEntry.DoesNotExist:
-            search_entry = SearchEntry(
-                content_type = content_type,
-                object_id = object_id,
-                object_id_int = object_id_int,
-            )
-        except SearchEntry.MultipleObjectsReturned:
-            # Oh no! Some non-transactional database has messed up!
-            search_entry = search_entries[0]
-            search_entries.exclude(id=search_entry.id).delete()
-        # Store data.
-        search_entry.engine_slug = self._engine_slug
-        search_entry.title = adapter.get_title(obj)
-        search_entry.description = adapter.get_description(obj)
-        search_entry.content = adapter.get_content(obj)
-        search_entry.url = adapter.get_url(obj)
-        search_entry.meta = adapter.get_meta(obj)
-        # Pass on the entry for final processing to the search backend.
-        get_backend().save_search_entry(search_entry, obj, adapter)
+        # Attempt to update the search entries.
+        update_count = search_entries.update(**search_entry_data)
+        if update_count == 0:
+            # This is the first time the entry was created.
+            search_entry_data.update((
+                ("content_type", content_type),
+                ("object_id", object_id),
+                ("object_id_int", object_id_int),
+            ))
+            yield SearchEntry(**search_entry_data)
+        elif update_count > 1:
+            # Oh no! Somehow we've got duplicated search entries!
+            search_entries.exclude(id=search_entries[0].id).delete()
+    
+    def update_obj_index(self, obj):
+        """Updates the search index for the given obj."""
+        _bulk_save_search_entries(list(self._update_obj_index_iter(obj)))
         
     # Signalling hooks.
             
@@ -429,12 +436,13 @@ class SearchEngine(object):
         """Signal handler for when a registered model has been saved."""
         if self._search_context_manager.is_active():
             self._search_context_manager.add_to_context(self, instance)
+        else:
+            self.update_obj_index(instance)
             
     def _pre_delete_receiver(self, instance, **kwargs):
         """Signal handler for when a registered model has been deleted."""
-        if self._search_context_manager.is_active():
-            _, search_entries = self._get_entries_for_obj(instance)
-            search_entries.delete()
+        _, search_entries = self._get_entries_for_obj(instance)
+        search_entries.delete()
         
     # Searching.
     
